#include "mainwindow.h"
#include "ui_mainwindow.h"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    LoginDialog* loginDialog = new LoginDialog( this );
    loginDialog->exec();

    if(loginDialog->result() == QDialog::Rejected){

        exit(1);
    }
    se3_session* tmp=loginDialog->getSession();
    memcpy(&s, tmp, sizeof(se3_session));
    if(L1_crypto_set_time(&s, (uint32_t)time(0))){

        exit(1);
    }
    if(secure_init(&s, -1, SE3_ALGO_MAX+1)){

        exit(1);
    }
    ui->setupUi(this);
    //connect (ui->actionSetEnvironment, SIGNAL(triggered(bool))), this, SLOT(dialogSetEnvironment()));
}

MainWindow::~MainWindow()
{
    secure_finit();
    if(s.logged_in)
        L1_logout(&s);
    delete ui;
}


bool MainWindow::manageOpenSaveFile(bool newfile){
    if(newfile)
        path = QFileDialog::getSaveFileName(this, "Save Image File", ".", "Image (*.jpg)");
    else
        path = QFileDialog::getOpenFileName(this, "Open Image File", ".", "Image (*.jpg)");

    if(path.isNull()) return false;

    filename = QFileInfo(QFile(path)).fileName();

    if(filename.isNull()) return false;
    path = path.left(path.size() - filename.size());
    return true;
}

void MainWindow::on_actionSetEnvironment_triggered()
{
    EnvironmentDialog *envDialog = new EnvironmentDialog(this, &s);
    envDialog->exec();
}


void MainWindow::on_close_btn_clicked()
{
    ui->plainName_lbl->setText("[PlainImage_FileName]");
    ui->encName_lbl->setText("[EncryptedImage_FileName]");
    ui->label->clear();
    filename.clear();

}

void MainWindow::on_openPlainImgbtn_clicked()
{
    uint16_t ret = SE3_OK;
    char enc_filename[65];
    uint16_t enc_len = 0;
    on_close_btn_clicked();
    if(!manageOpenSaveFile(false)) return;  // set path of file


    QImage *plain_img = new QImage(path + filename);

    QPixmap p = QPixmap::fromImage(*plain_img);

    ui->plainName_lbl->setText(filename);

    memset(enc_filename, 0, 65);
    crypto_filename(filename.toUtf8().data(), enc_filename, &enc_len );
    ui->encName_lbl->setText(QString(enc_filename));

    int w = ui->label->width();
    int h = ui->label->height();

    //ui->label->setText("Loading...");
    // set a scaled pixmap to a w x h window keeping its aspect ratio
    ui->label->setPixmap(p.scaled(w,h,Qt::KeepAspectRatio));
    ui->label->show();

    // Begin encryption of file
    QString *path_plainfile = new QString(path + filename);
    if((ret = secure_open(path_plainfile->toUtf8().data(), &sefile_file, SEFILE_WRITE, SEFILE_NEWFILE))){
        if ( ret == SEFILE_SIGNATURE_MISMATCH ){   // Check if signature on header failed
            ui->plainName_lbl->setText("Signature Mismatch");
            ui->encName_lbl->setText("Signature Mismatch");
        }else{
            ui->plainName_lbl->setText("Error code: " + ret);
            ui->encName_lbl->setText("Error code: " + ret);
        }
        return; // todo error
    }
    QFile plain_file(path+filename);
    plain_file.open(QIODevice::ReadOnly);
    qint64 size_len = plain_file.size();

    uint8_t *buffer = (uint8_t*) malloc((size_t)size_len*sizeof(uint8_t));
    memcpy(buffer, plain_file.readAll().data(), (size_t)size_len);
    if((ret = secure_write(&sefile_file, buffer, size_len))) {
        if ( ret == SEFILE_SIGNATURE_MISMATCH ){
            ui->plainName_lbl->setText("Signature Mismatch");
            ui->encName_lbl->setText("Signature Mismatch");
        }else{
            ui->plainName_lbl->setText("Error code: " + ret);
            ui->encName_lbl->setText("Error code: " + ret);
        }
    }

    free(buffer);
    secure_close(&sefile_file);
    plain_file.close();
    return; // todo error
    // --
}

void MainWindow::on_openEncImgbtn_clicked()
{
    QByteArray plain_buffer;
    uint8_t *buffer;
    char enc_filename[65];
    uint16_t ret = SE3_OK;
    uint16_t enc_len = 0;
    uint32_t nBytesRead = 0, file_len = 0;
    on_close_btn_clicked();
    SecureFileDialog *fileDialog = new SecureFileDialog( this, 0 );
    fileDialog->exec();
    if(fileDialog->result()==QDialog::Rejected){
        return ;
    }
    path = fileDialog->getChosenFile();
    filename = QFileInfo(QFile(path)).fileName();

    if(filename.isNull()) return;
    path = path.left(path.size() - filename.size());

    QString *path_plainfile = new QString(path + filename);
    if((ret = secure_open(path_plainfile->toUtf8().data(), &sefile_file, SEFILE_WRITE, SEFILE_OPEN))){
        if ( ret == SEFILE_SIGNATURE_MISMATCH ){   // Check if signature on header failed
            ui->plainName_lbl->setText("Signature Mismatch");
            ui->encName_lbl->setText("Signature Mismatch");
        }else{
            ui->plainName_lbl->setText("Error code: " + ret);
            ui->encName_lbl->setText("Error code: " + ret);
        }
        return; // todo error
    }
    if((ret = secure_getfilesize(path_plainfile->toUtf8().data(), &file_len))){
        if ( ret == SEFILE_SIGNATURE_MISMATCH ){
            ui->plainName_lbl->setText("Signature Mismatch");
            ui->encName_lbl->setText("Signature Mismatch");
            secure_close(&sefile_file);
            return;
        }else if (ret > 0){
            ui->plainName_lbl->setText("Error code: " + ret);
            ui->encName_lbl->setText("Error code: " + ret);
            secure_close(&sefile_file);
            return;
        }
    }
    buffer = (uint8_t *)calloc(file_len, sizeof(uint8_t));
    if((ret = secure_read(&sefile_file,  buffer, file_len, &nBytesRead))){
        if ( ret == SEFILE_SIGNATURE_MISMATCH ){
            ui->plainName_lbl->setText("Signature Mismatch");
            ui->encName_lbl->setText("Signature Mismatch");
            secure_close(&sefile_file);
            return;
        }else if (ret > 0){
            ui->plainName_lbl->setText("Error code: " + ret);
            ui->encName_lbl->setText("Error code: " + ret);
            secure_close(&sefile_file);
            return;
        }
    }
    plain_buffer = QByteArray((char *)buffer, (int)nBytesRead);
    //    QImage *plain_img = new QImage(path + filename);

    QPixmap p;
    p.loadFromData(plain_buffer);
    ui->plainName_lbl->setText(filename);

    memset(enc_filename, 0, 65);
    crypto_filename(filename.toUtf8().data(), enc_filename, &enc_len );
    ui->encName_lbl->setText(QString(enc_filename));

    int w = ui->label->width();
    int h = ui->label->height();

    //ui->label->setText("Loading...");
    // set a scaled pixmap to a w x h window keeping its aspect ratio
    ui->label->setPixmap(p.scaled(w,h,Qt::KeepAspectRatio));
    ui->label->show();

}
